/////////////////////////////////////////////////////////////////////
///   devnetX Quad Art-Net Node (DMX Output Firmware), Ver. 1.0   ///
///   Serial Peripheral Interface Direct Memory Access Module     ///
///   ---------------------------------------------------------   ///
///   Author: Alexej Goujine                                      ///
///   Copyright  2011 - 2013 devnetX, http://www.devnetx.at      ///
///   E-Mail: dev@devnetx.at                                      ///
///   Last changed on: 28.11.2013 (DD.MM.YYYY)                    ///
///   ---------------------------------------------------------   ///
///   This code is provided 'as is', use it at you own risk.      ///
/////////////////////////////////////////////////////////////////////

#include <p18cxxx.h>
#include <GenericTypeDefs.h>
#include <string.h>
#include "SPIDMA.h"
#include "DMX512.h"

// Variables
#pragma udata SPIDMA
volatile DMXPACKET SPIRXPacket;
volatile DMXPACKET SPITXPacket;

#pragma udata
volatile DMXPACKET *SPIRXPacketPointer;         // Pointer to SPIRXPacket
volatile DMXPACKET *SPITXPacketPointer;         // Pointer to SPITXPacket

// Initialize the SPI DMA module as slave
void SPIDMASlaveInit(void)
{
    // Init Pointers
    SPIRXPacketPointer = &SPIRXPacket;
    SPITXPacketPointer = &SPITXPacket;
    
    // Clear Buffers
    memset(SPIRXPacketPointer, 0x00, sizeof(SPIRXPacket));

    // Set TX buffer confirmation bytes (not used yet, half duplex mode)
    memset(SPITXPacketPointer, 0x55, sizeof(SPITXPacket));

    // PPSCON Unlock sequence consists of writing 0x55 and 0xAA to the EECON2 register.
    _asm
        MOVLW 0x55
        MOVWF EECON2, 0
        MOVLW 0xAA
        MOVWF EECON2, 0
    _endasm

    // ========================
    // MSSP2 SPIDMA PIN MAPPING
    // ========================
    // SDI2 = RC0 (RP11)
    // SDO2 = RC1 (RP12)
    // SCK2 = RC2 (RP13)
    // SS2  = RC3 (RP14)
    // ========================

    PPSCONbits.IOLOCK = 0;      // Unlock write to RPINRx and RPORx registers
        RPINR21 = 0x0B;         // Assign SDI2 input to pin RP11 (0x0B)
        RPOR12  = 0x09;         // Assign SDO2 output (0x09) to the RP12
        RPOR13  = 0x0A;         // Assign SCK2 output (0x0A) to pin RP13
        RPINR22 = 0x0D;         // Assign SCK2 input to pin RP13 (0x0D)
        RPINR23 = 0x0E;         // Assign SS2  input to pin RP14 (0x0E)
    PPSCONbits.IOLOCK = 1;      // Lock the PPS registers to prevent changes

    // Init SPI Pins
    LATCbits.LATC1 = 0;         // Initialize SDO2 pin to an idle state
    TRISCbits.TRISC0 = 1;       // Make SDI2 input
    TRISCbits.TRISC1 = 0;       // Make SDO2 output
    TRISCbits.TRISC2 = 1;       // Make SCK2 input
    TRISCbits.TRISC3 = 1;       // Make SS2  input

    // Init MSSP2
    SPIDMA_INTERRUPT = 0;       // Clear Interrupt Flag
    SSP2STATbits.CKE = 1;       // Transmit occurs on transition from active to Idle clock state
    SSP2STATbits.SMP = 0;       // Input sampled at middle of data output time
    SSP2CON1 = 0b00100100;      // SPI Slave Mode, /SS pin control enabled, SPI: ENABLED

    // Init SPIDMA
    //DMACON1 = 0b00111010;     // RXINC = 1, DLYINTEN = 1, Full-Duplex
    DMACON1 = 0b00110010;       // RXINC = 1, DLYINTEN = 1, Half-Duplex
    DMACON2 = 0b11110000;       // Minimum delay between bytes, interrupt only once when the transaction is complete

    // SPI DMA Interrupts
    IPR3bits.SSP2IP = 0;        // SPI Interrupt Priority: LOW
    PIR3bits.SSP2IF = 0;        // Clear Interrupt Flag
    PIE3bits.SSP2IE = 1;        // Enable SPI Interrupt
}

// Start the SPI DMA reception
void SPIDMAStart(void)
{
    // Clear Interrupt Flag
    SPIDMA_INTERRUPT = 0;

    // Prepare RX & TX (DMABC is zero-based)
    RXADDRH = HIGH((UINT16)SPIRXPacketPointer);
    RXADDRL = LOW ((UINT16)SPIRXPacketPointer);
    TXADDRH = HIGH((UINT16)SPITXPacketPointer);
    TXADDRL = LOW ((UINT16)SPITXPacketPointer);
    DMABCH  = HIGH(sizeof(SPIRXPacket) - 1);
    DMABCL  = LOW (sizeof(SPIRXPacket) - 1);

    // Start SPI DMA
    DMACON1bits.DMAEN = 1;
}

// End of the SPI DMA reception
void SPIDMAInterrupt(void)
{
    if (SPIDMA_INTERRUPT)
    {
        // Clear Interrupt flag and start new receptipon
        SPIDMA_INTERRUPT = 0;
        SPIDMAStart();
    }
}
