/////////////////////////////////////////////////////////////////////
///   devnetX Quad Art-Net Node (DMX Output Firmware), Ver. 1.0   ///
///   Main Module                                                 ///
///   ---------------------------------------------------------   ///
///   Author: Alexej Goujine                                      ///
///   Copyright  2011 - 2013 devnetX, http://www.devnetx.at      ///
///   E-Mail: dev@devnetx.at                                      ///
///   Last changed on: 27.11.2013 (DD.MM.YYYY)                    ///
///   ---------------------------------------------------------   ///
///   This code is provided 'as is', use it at you own risk.      ///
/////////////////////////////////////////////////////////////////////

#include <p18cxxx.h>
#include <GenericTypeDefs.h>
#include "DMX512.h"
#include "SPIDMA.h"

// Private Prototypes
extern void _startup (void);
void HighPriorityISR();
void LowPriorityISR();

// Vector Remapping
#pragma code REMAPPED_RESET_INTERRUPT_VECTOR = 0x1000
    void _reset (void)              { _asm goto _startup _endasm }
#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = 0x1008
    void Remapped_High_ISR (void)   { _asm goto HighPriorityISR _endasm }
#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = 0x1018
    void Remapped_Low_ISR (void)    { _asm goto LowPriorityISR _endasm }
#pragma code HIGH_INTERRUPT_VECTOR = 0x08
    void High_ISR (void)            { _asm goto 0x1008 _endasm }
#pragma code LOW_INTERRUPT_VECTOR = 0x18
    void Low_ISR (void)             { _asm goto 0x1018 _endasm }
#pragma code

// HIGH Priority Interrupts
#pragma interrupt HighPriorityISR
void HighPriorityISR()
{
    DMX512Interrupt();
}

// LOW Priority Interrupts
#pragma interruptlow LowPriorityISR
void LowPriorityISR()
{
    SPIDMAInterrupt();
    DMX512LEDInterrupt();
}

// Main App Start
#pragma code
void main(void)
{
    // Configure Oscillator
    OSCCON = 0x00;
    OSCTUNE = 0x40;
    REFOCON = 0x00;

    // Configure AD Converter (OFF)
    ADCON0 = 0x00;
    ADCON1 = 0x0f;
    ANCON0 = 0xff;
    ANCON1 = 0x1f;

    // Configure Comparators (OFF)
    CM1CON = 0x00;
    CM2CON = 0x00;

    // Init SPIDMA
    SPIDMASlaveInit();

    // Init DMX512
    DMX512Init();

    // Init Global Interrupts
    RCONbits.IPEN = 1;          // Interrupt priorities: ENABLED
    INTCONbits.GIEL = 1;        // All low-priority peripheral interrupts: ENABLED
    INTCONbits.GIEH = 1;        // All high-priority interrupts: ENABLED

    // Start SPI DMA reception
    SPIDMAStart();

    while (1)
    {
        Nop();                  // Nothing to do during idle
    }
}
